/* serial1.c - Kanonikus soros port kezeles. */

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <termios.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>

#define BAUDRATE B38400      
/* Az erteket a hasznalt soros port eszkoz allomanyara kell 
   modositani. */
#define MODEMDEVICE "/dev/ttyS1"

#define FALSE 0
#define TRUE 1

volatile int STOP=FALSE; 

int main(void)
{
  int fd,res;
  struct termios oldtio,newtio;
  char buf[255];

  /* Megnyitjuk a modemet olvasasra es irasra. A tty kontrollt 
     kikapcsoljuk, hogy a CTRL-C karakterre ne szakadjon meg a 
     program futasa. */
  fd = open(MODEMDEVICE, O_RDWR | O_NOCTTY ); 
  if(fd < 0) { perror(MODEMDEVICE); exit(-1); }

  /* Elmentjuk a soros port aktualis beallitasait. */
  tcgetattr(fd, &oldtio);
  /* Inicializaljuk a strukturat az uj beallitasokhoz. */
  bzero(&newtio, sizeof(newtio));

  /* 
  BAUDRATE: a soros port sebessege
  CRTSCTS : a port hardver atvitel szabalyozasa
  CS8     : 8n1 (8 bit, nincs paritas,1 stopbit)
  CLOCAL  : lokalis kapcsolat, nincsenek modem jelek
  CREAD   : a karakterek fogadasanak bekapcsolasa
  */
  newtio.c_cflag = BAUDRATE | CRTSCTS | CS8 | CLOCAL | CREAD;

  /*
  IGNPAR  : a paritas hibas bajtokat figyelmen kivul hagyja
  ICRNL   : a CR karaktert NL karakterre konvertalja
  */
  newtio.c_iflag = IGNPAR | ICRNL;

  /* Nyers kimenet beallitasa. */
  newtio.c_oflag = 0;

  /* Engedelyezi a kanonikus bemenetet, kikapcsolja a karakter 
     visszajelzest, es letiltja a szignalok kuldeset. */
  newtio.c_lflag = ICANON;

  /* A kontroll karakterek inicializalasa. */
  newtio.c_cc[VDISCARD] = 0;     /* Ctrl-u */
  newtio.c_cc[VEOF]     = 4;     /* Ctrl-d */
  newtio.c_cc[VEOL]     = 0;     /* '\0' */
  newtio.c_cc[VEOL2]    = 0;     /* '\0' */
  newtio.c_cc[VERASE]   = 0;     /* del */
  newtio.c_cc[VINTR]    = 0;     /* Ctrl-c */ 
  newtio.c_cc[VKILL]    = 0;     /* @ */
  newtio.c_cc[VLNEXT]   = 0;     /* Ctrl-v */
  newtio.c_cc[VQUIT]    = 0;     /* Ctrl-\ */
  newtio.c_cc[VREPRINT] = 0;     /* Ctrl-r */
  newtio.c_cc[VSWTC]    = 0;     /* '\0' */
  newtio.c_cc[VSTART]   = 0;     /* Ctrl-q */ 
  newtio.c_cc[VSTOP]    = 0;     /* Ctrl-s */
  newtio.c_cc[VSUSP]    = 0;     /* Ctrl-z */
  newtio.c_cc[VWERASE]  = 0;     /* Ctrl-w */

  newtio.c_cc[VTIME]    = 0;     /* karakterek kozti idozito */
  newtio.c_cc[VMIN]     = 1;     /* blokkolja az olvasast, amig 
                                    nem erkezik egy karakter */

  /* Toroljuk soros port buffereit es aktivaljuk a beallitasokat. 
  */
  tcflush(fd, TCIFLUSH);
  tcsetattr(fd, TCSANOW, &newtio);
  
  /* A beallitasok megvannak, kezeljuk a bemenetet. */
  while(STOP == FALSE) /* A kilepesig tart a ciklus. */
  {
    res = read(fd, buf, 255); 
    buf[res] = 0;
    printf(":%s:%d\n", buf, res);
    if(buf[0] == 'z') STOP=TRUE;
  }
  /* Visszaallitjuk a regi port beallitasokat. */
  tcsetattr(fd, TCSANOW, &oldtio);

  return 0;
}
