/* utas.c - Pelda szemaforok hasznalatara.
   Utasok bejelentkezese repteri pultnal. */

#include <stdlib.h>
#include <sys/types.h>
#include <sys/ipc.h>
#include <sys/sem.h>
#include <stdio.h>
#include <values.h>
#include "kozos.h"
#include "errno.h"

int semopen(key_t key);
int semgetval(int semid, int member);
int semacquirewait(int semid,int member);
int semacquire(int semid, int member);
void semrelease(int semid, int member);
int semismember(int semid,int member);
int semgetwaitingprocs(int semid,int member);

int main(int argc, char*argv[])
{
  key_t key;
  int semid;
  int i;
  
  int semncntmin=MAXINT, semncnt;
  int semminmember=0;
  int locked=-1;

  if(argc!=2)
  {
    fprintf(stderr,"Hasznalat: %s <legitarsasag azonosito>\n", argv[0]);
    return 1;
  }
 
  key=atoi(argv[1]);

  /* Kapcsolodas az elso program parameter altal megadott szemaforhoz. */
  if((semid=semopen(key)) == -1)
  {
    /* Ha nincs ilyen szemafortomb... */
    fprintf(stderr,"A legitarsasag nem letezik!\n");
    return 1;
  }      

  for(i=0;i<SEM_MAX_IN_SET;i++)
  {
    int ret=semacquire(semid,i);
    if(ret==0)
    {
      locked=i;
      printf("Az utast a %d. pultnal szolgaljak ki.\n",i);
      break;
    }
    else if(ret==-1) /* Hiba: tulfut a ciklus */
    {
      break;
    }
    else if(ret==1) /* Nincs szabad pult */
    {
      semncnt=semgetwaitingprocs(semid,i);
      if(semncnt<semncntmin)
      {
        semncntmin=semncnt;
        semminmember=i;
      }
      printf("A %d. pultra %d utas varakozik.\n",i,semncnt);
    }
  }

  if(locked==-1)/* Ha nincs szabad pult...*/
  {
    /* Beallunk a legrovidebb sorba */
    printf("A legrovidebb sor a %d. pultnal van %d utassal.\n",semminmember,semncntmin);
    semacquirewait(semid,semminmember);
    locked=semminmember;
  }

  printf("Nyomj <Enter>-t a pult elhagyasahoz!\n");
  getchar();
  semrelease(semid,locked);

  return 0;
}

/* Kapcsolodas egy, mar letezo szemaforhoz. Ha nincs ilyen, a visszateresi
   ertek -1. */
int semopen(key_t key)
{
  return semget(key, 0, 0666);
}

/* Lekerdezi a szemafor aktualis erteket */ 
int semgetval(int semid, int member)
{
  int semval;
        
  semval = semctl(semid, member, GETVAL, 0);
  return semval ;
}

/* A megadott szemafortomb a member argumentumban megadott indexu tagjat eggyel
   csokkenti. Nem varakozik. */
int semacquire(int semid,int member)
{
  struct sembuf sem_acquire={ member, -1, IPC_NOWAIT};

  
  if((semop(semid, &sem_acquire, 1)) == -1)
  {
    if(errno == EAGAIN)
    {
     printf("A %d. pult nem all rendelkezesre(%d).\n",member,semgetval(semid,member));
     return 1;
    }
    else
    {
     fprintf(stderr, "Muszaki hiba a %d. pultnal.\n",member);
     return -2;
    }
  }
  else
  {
    printf("Utas a %d. pultnal.\n",member);
  }
  return 0;
}

/* A megadott szemafortomb a member argumentumban megadott indexu tagjat eggyel
   csokkenti. Varakozik. */
int semacquirewait(int semid,int member)
{
  struct sembuf sem_acquire={ member, -1, 0};

  
  
  /* A szemafor lefoglalasa */
  
  printf("Az utas beallt a %d. pultnal allo sorba.\n",member);
  if((semop(semid, &sem_acquire, 1)) == -1)
  {
    fprintf(stderr, "Muszaki hiba a %d. pultnal.\n",member);
    return -2;
  }
  else
  {
    printf("A %d. pult felszabadult. Az utast kiszolgaljak.\n",member);
  }

  
  return 0;
}


/* A szemafortomb member argumentumban megadott indexu elemenek erteket novel
 eggyel. */
void semrelease(int semid, int member)
{
  struct sembuf sem_release={ member, 1, IPC_NOWAIT};
  int semval;
  
  /* Foglalja valaki a szeamafort? */
  semval = semgetval(semid, member);
  if(semval == SEM_RESOURCE_INIT) 
  {
    fprintf(stderr, "A szemafor nincs lefoglalva!\n");
    exit(1);
  }

  /* Attempt to lock the semaphore set */
  if((semop(semid, &sem_release, 1)) == -1)
  {
    fprintf(stderr, "Az elengedes sikertelen!\n");
    exit(1);
  }
  else
  {
    printf("Az utas elhagyta a %d. pultot.\n",member);
  }        
}

/* Megallapitja, hogy a semid szemafortombnek letezik-e member indexu tagja.
   Ha igen, a visszateresi ertek 0. */
int semismember(int semid,int member)
{
  int membernum;
  union semun semopts;

  struct semid_ds mysemds;
  semopts.buf = &mysemds;

  /* Szemafor informacio lekerdezese */
  semctl(semid,0,IPC_STAT,semopts);

  /* A szemafortomb tagjainak szama */
  membernum=semopts.buf->sem_nsems;

  if( member<0 || member>(membernum-1))
  {
    return 0;
  }
  else
  {
    return -1;
  }
}

/* Visszaadja a varakozo folyamatok szamat */
int semgetwaitingprocs(int semid,int member)
{
  int semncnt= semctl(semid, member, GETNCNT, 0);
  return semncnt;	
}
