#include <sys/types.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <errno.h>
#include <unistd.h>
#include <syslog.h>
#include <stdbool.h>
#include <signal.h>
#include <string.h>
#include <time.h>
#include <sys/sendfile.h>

#define DAEMON_NAME "backupd"
#define BACKUP_TIME 60

bool stop = false;
int counter = 0;

void signal_handler(int sig) {
 
    switch(sig) {
        case SIGTERM:
	    stop = true;
            syslog(LOG_WARNING, "Received SIGTERM signal.");
            break;
        default:
            syslog(LOG_WARNING, "Unhandled signal (%d) %s",sig, strsignal(sig));
            break;
    }
}

void backup(char* file, char* backup)
{
	int file_fd;
 	int backup_fd;
 	struct stat stat_buf;
 	off_t offset = 0;

 	file_fd = open (file, O_RDONLY);

	if(file_fd < 0)
	{
		syslog(LOG_ERR, "Hiba a bemeneti allomany megnyitasa kozben.");
		return;
	}
	fstat (file_fd, &stat_buf);
	
	if(time(NULL) - stat_buf.st_mtime < BACKUP_TIME)
	{
		close (file_fd);
		return;
	}

	backup_fd = open (backup, O_WRONLY | O_CREAT, stat_buf.st_mode);
	
	if(backup_fd < 0)
	{
		syslog(LOG_ERR, "Hiba a biztonsagi allomany megnyitasa kozben.");
		close (file_fd);
		return;
	}
	

	sendfile (backup_fd, file_fd, &offset, stat_buf.st_size);

	close (file_fd);
	close (backup_fd);

}

int main(int argc, char*argv[]) 
{
	if(argc < 3)
	{
		printf("%d\n", argc);
		printf("Usage: backupd <file> <backupfile>\n\n");
		return -1;
	}
                
        pid_t pid, sid;
        

        // Leforkolunk a szulotol
        pid = fork();
        if (pid < 0) 
	{
		fprintf(stderr, "Hiba az uj folyamat letrehozasa kozben.");		
                exit(EXIT_FAILURE);
        }
        // Ha sikerult a fork, nincs szukseg a szulore.
        if (pid > 0) 
	{
                exit(EXIT_SUCCESS);
        }

	signal(SIGTERM, signal_handler);


        umask(0);
                
        // Log megnyitasa
        setlogmask(LOG_UPTO(LOG_INFO));
        openlog(DAEMON_NAME, LOG_CONS, LOG_USER);     
        syslog(LOG_INFO, "Backupd elindult.");
        
        // Uj, terminal nelkuli munkamenet a processznek
        sid = setsid();
        if (sid < 0) 
	{
		syslog(LOG_ERR, "Hiba a munkamenet letrehozasa kozben.");		
	      	exit(EXIT_FAILURE);
        }
        

        // A munkakonyvtar beallitasa
        if ((chdir("/")) < 0) 
	{
                syslog(LOG_ERR, "Hiba a munkakonyvtar beallitasa kozben.");		
                exit(EXIT_FAILURE);
        }
        
        // A szabvany leirok lezarasa
        close(STDIN_FILENO);
        close(STDOUT_FILENO);
        close(STDERR_FILENO);

 	// A demon feladatainak elvegzese
 	while(!stop)
	{
		sleep(2);
		counter++;
		if(counter == BACKUP_TIME/2)
		{
			counter = 0;
			backup(argv[1], argv[2]);
		}
	}
   	syslog(LOG_INFO, "Backupd leallt.");
   	exit(EXIT_SUCCESS);
}
