/* fork3.c - Pelda a fork() hasznalatara. 
   Letrehozunk ket gyere processzt, amelyek nevtelen csovezetekeken keresztul
   kuldik vissza a fo processznek a szamitasaik eredmenyet. */
 
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <time.h>
#include <sys/wait.h>
#include <fcntl.h>
#include <string.h>

/* A gyerek processz fo fuggvenye. */
void child(int fd, int rnd)
{
  int i;
  int pid;
  /* Az adatok kozotti varakozas. */
  unsigned long w;
  /* A szoveg osszeallitasahoz a buffer. */
  char buf[80];

  /* A veletlen szambol kiszamoljuk a varakozast. */
  w=(rnd%1000+1)*1000;

  pid=getpid();

  /* Ciklus amelyben megfelelo idokozonkent informaciot kuldunk a
     fo processznek. */
  for(i=1; i<=20; i++)
  {
    sprintf(buf,"%d ", i);
    write(fd, buf, strlen(buf));
    usleep(w);
  }
  close(fd);
}

int main()
{
  int cpid1,cpid2;
  int fd[2][2];
  int rnd1,rnd2;
  char buf[80];
  int i, rc, maxfd;
  fd_set watchset;
  fd_set inset;

  printf("Start\n");

  /* A ket gyerek processz szamara veletlen szamok. */
  srand(time(NULL));
  rnd1=rand();
  rnd2=rand();

  /* A ket nevtelen csovezetek letrehozasa. */
  pipe(fd[0]);
  pipe(fd[1]);

  /* Az 1. gyerek processz letrehozasa. */	
  cpid1=fork();	
  if(cpid1<0)
  {
    printf("Fork error\n");
    exit(-1);
  }
  if(cpid1==0)
  {
    //child1
    close(fd[0][0]);
    child(fd[0][1], rnd1);
    return 0;
  }
  close(fd[0][1]);

  /* A 2. gyerek processz letrehozasa. */	
  cpid2=fork();	
  if(cpid2<0)
  {
    printf("Fork error\n");
    exit(-1);
  }
  if(cpid2==0)
  {
    //child2
    close(fd[1][0]);
    child(fd[1][1], rnd2);
    return 0;
  }
  close(fd[1][1]);

  /* A szulo processzbol select() metodussal parhuzamosan olvassuk
     a ket gyerek altal kuldott adatokat. */
  FD_ZERO(&watchset);
  FD_SET(fd[0][0], &watchset);
  FD_SET(fd[1][0], &watchset);
  maxfd = fd[0][0] > fd[1][0] ? fd[0][0] : fd[1][0];

  /* Addig tart a ciklus, amig valamelyik csovezetek nyitva van. */
  while(FD_ISSET(fd[0][0], &watchset) ||
    FD_ISSET(fd[1][0], &watchset))
  {
    inset=watchset;
    if(select(maxfd+1, &inset, NULL, NULL, NULL) < 0)
    {
      perror("select");
      waitpid(cpid1,NULL,0);
      waitpid(cpid2,NULL,0);
      return -1;
    }

    /* Beolvassuk es kiirjuk a ket csovezeteken erkezo adatokat. */
    for(i=0; i<2; i++)
    {
      if(FD_ISSET(fd[i][0], &inset))
      {
        rc=read(fd[i][0], buf, sizeof(buf));
        if(rc < 0)
        {
          sprintf(buf, "read%d", i+1);
          perror(buf);
          close(fd[i][0]);
          FD_CLR(fd[i][0], &watchset);
        }
        else if(rc == 0)
        {
          close(fd[i][0]);
          FD_CLR(fd[i][0], &watchset);
        }
        else
        {
          buf[rc]='\0';
          printf("pipe%d: %s\n", i+1, buf);
        }
      }
    }
  }

  /* Bevarjuk a gyerek processzeket. */
  waitpid(cpid1,NULL,0);
  waitpid(cpid2,NULL,0);

  return 0;
}
