/* chatszerver.c
 * 
 * Egyszálú, poll alapú TCP szerver példa.
 * A példaprogram egy csevegő szervert valósít meg, amely fogadja a
 * kliensek kapcsolódásait, majd az egyik kapcsolaton érkező szöveget
 * szétküldi a többi kapcsolatra.
 * Tesztelés: telnet <hoszt> 3344
 * 
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <poll.h>

#define PORT 3344
#define MAXCONNS 5
#define FULLMSG "Megtelt!"

int ssock;
int connect_list[MAXCONNS];
struct pollfd poll_list[MAXCONNS + 1];

int build_poll_list()
{
  int i, count;
  
  poll_list[0].fd = ssock;
  poll_list[0].events = POLLIN;
  
  count = 1;
  for(i = 0; i < MAXCONNS; i++)
  {
    if(connect_list[i] >= 0)
    {
      poll_list[count].fd = connect_list[i];
      poll_list[count].events = POLLIN;
      count++;
    }
  }
  return count;
}

void handle_new_connection()
{
  int i, csock;
  
  csock = accept(ssock, NULL, NULL);
  if(csock < 0) return;
  for(i = 0; i < MAXCONNS; i++)
  {
    if(connect_list[i] < 0)
    {
      connect_list[i] = csock;
      csock = -1;
      break;
    }
  }
  if(csock >= 0)
  {
    send(csock, FULLMSG, strlen(FULLMSG), 0);
    close(csock);
  }
}

void process_read(int csock)
{
  char buf[256];
  int len;
  int i;
  
  len = recv(csock, buf, sizeof(buf), 0);
  if(len > 0)
  {
    for(i = 0; i < MAXCONNS; i++)
    {
      if((connect_list[i] >= 0) && (connect_list[i] != csock))
      {
	send(connect_list[i], buf, len, 0);
      }
    }
  }
}

int main() 
{
  struct sockaddr_in6 addr;
  int reuse;
  int i;
  int ii;
  
  if((ssock = socket(PF_INET6, SOCK_STREAM, 0)) < 0)
  {
   perror("socket"); 
   return 1;
  }
  
  reuse = 1;
  setsockopt(ssock, SOL_SOCKET, SO_REUSEADDR, &reuse, sizeof(reuse));
  
  memset(&addr, 0, sizeof(addr));
  addr.sin6_family = AF_INET6;
  addr.sin6_addr = in6addr_any;
  addr.sin6_port = htons(PORT);
  
  if(bind(ssock, (struct sockaddr*)&addr, sizeof(addr)) < 0)
  {
    perror("bind");    
    return 1;
  }
  
  if(listen(ssock, 5) < 0)
  {
    perror("listen");
    return 1;
  }
  
  for(i = 0; i< MAXCONNS; i++) connect_list[i] = -1;
  
  while(1)
  {
    int count = build_poll_list();
    if(poll(poll_list, count, -1) > 0)
    {
      if(poll_list[0].revents & POLLIN)
      {
	handle_new_connection();
      }
      for(i = 1; i < count; i++)
      {
	if(poll_list[i].revents & (POLLERR | POLLHUP))
	{
	    for(ii = 0 ; ii < MAXCONNS; ii++)
	    {
	      if(connect_list[ii] == poll_list[i].fd)
	      {
		connect_list[ii] = -1;
	      }
	    }
	    close(poll_list[i].fd);
	}
	else if(poll_list[i].revents & POLLIN)
	{
	  process_read(poll_list[i].fd);
	} 
      }
    }
  }
  
  return 0;
}
