/* helloproc2.c
 * 
 * Egyszerű kernel modul amely a /proc állományrendszerben létrehoz egy
 * olvasható és írható állományt.
 * 
 */

#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/proc_fs.h>
#include <asm/uaccess.h>
#include <linux/mutex.h>

#define GBUFFERSIZE 256
static char gbuffer[GBUFFERSIZE];
static unsigned long gbufferlen = 0;

static DEFINE_MUTEX(lock);

/* Az állomány olvasását kezelő függvény. */
int procfile_read(char* buffer, char** buffer_location, off_t offset, int
buffer_length, int *eof, void *procdata)
{
  int len;

  /* Az egyszerűség kedvéért nem engedünk részleges olvasást. */        
  if(offset)
  {
    *eof = 1;
    return 0;
  }

  /* Feltöltjük a buffert és visszatérünk az adat hosszával. */
  if(mutex_lock_interruptible(&lock)) return -ERESTARTSYS;
  memcpy(buffer, gbuffer, gbufferlen);
  len = gbufferlen;
  mutex_unlock(&lock);

  return len;
}

/* Az állomány írását kezelő függvény. */
int procfile_write(struct file *pfile, const char *buffer, unsigned long count,
void *procdata)
{
  if(count > GBUFFERSIZE)
  {
    count = GBUFFERSIZE;
  }

  /* Átmásoljuk az adatokat a kernel címterületről a felhasználói területre. */
  if(mutex_lock_interruptible(&lock)) return -ERESTARTSYS;
  if(copy_from_user(gbuffer, buffer, count))
  {
    gbufferlen = 0;
    mutex_unlock(&lock);
    return -EFAULT;
  }
  gbufferlen = count;
  mutex_unlock(&lock);

  /* Visszatérünk a visszaadott byte-ok számával. */
  return count;
}

static int __init hello_init(void)
{
  struct proc_dir_entry *hello_proc_ent;

  /* Dinamikusan hozzuk létre a proc állomány bejegyzést. */
  hello_proc_ent = create_proc_entry("hello", S_IFREG | S_IRUGO | S_IWUGO, 
    0);

  if(!hello_proc_ent)
  {
    remove_proc_entry("hello", 0);
    printk("Error: A /proc/hello nem letrehozhato.\n");
    return -ENOMEM;
  }
  
  hello_proc_ent->nlink = 1;
  hello_proc_ent->read_proc = procfile_read;
  hello_proc_ent->write_proc = procfile_write;
  hello_proc_ent->mode = S_IFREG | S_IRUGO | S_IWUGO;
  hello_proc_ent->uid = 0;
  hello_proc_ent->gid = 0;
  hello_proc_ent->size = 256;
  printk("A /proc/hello letrehozva.\n");
  
  return 0;
}

static void __exit hello_exit(void)
{
  /* Megsemmisítjük a proc állomány bejegyzést. */
  remove_proc_entry("hello", 0);
  printk("A /proc/hello eltavolitva.\n");
}

module_init(hello_init);
module_exit(hello_exit);

MODULE_DESCRIPTION("Hello proc");
MODULE_LICENSE("GPL");
