/* poll-driver.c
 * 
 * A poll (select) implementációját bemutató példa.
 * 
 */

#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/fs.h>
#include <asm/uaccess.h>
#include <linux/poll.h>
#include <linux/sched.h>
#include <linux/slab.h>
#include <linux/device.h>

#define DEVICE_NAME "hello"
static int major_num=120;
#define CLASS_NAME "helloclass"
static struct class* hello_class;

DECLARE_WAIT_QUEUE_HEAD(wait_queue);
int counter=0;
static DEFINE_SPINLOCK(lock);

static ssize_t hello_write(struct file *pfile, const char *buf, size_t count, loff_t *ppos)
{
  int lcounter;
  printk("Increase counter..\n");
  spin_lock(&lock);
  counter += count;
  lcounter = counter;
  spin_unlock(&lock);
  printk("Value: %d\n", lcounter);
  wake_up_interruptible(&wait_queue);
  return count;
}

static ssize_t hello_read(struct file *pfile, char *buf, size_t count, loff_t *ppos)
{
  int lcounter;
  printk("Sleep until counter reach 10...\n");
  wait_event_interruptible(wait_queue, (counter >= 10));
  printk("End of sleep\n");
  spin_lock(&lock);
  counter -= 10;
  lcounter = counter;
  spin_unlock(&lock);
  printk("Value: %d\n", lcounter);
  return 0;
}

/* A poll függvény implementációja. */
unsigned int hello_poll(struct file *pfile, poll_table *wait)
{
  unsigned int mask = 0;

  poll_wait(pfile, &wait_queue,  wait);
  
  spin_lock(&lock);
  if(counter >= 10) mask |= POLLIN | POLLRDNORM;
  spin_unlock(&lock);

  return mask;
}

static int hello_open(struct inode *inode, struct file *pfile)
{
  try_module_get(THIS_MODULE);
  printk(DEVICE_NAME " open.\n");
  return 0;
}

static int hello_close(struct inode *inode, struct file *pfile)
{
  printk(DEVICE_NAME " close.\n");
  module_put(THIS_MODULE);
  return 0;
}

static struct file_operations hello_fops =
{
  owner:	THIS_MODULE,
  read:	hello_read,
  write:	hello_write,
  open:	hello_open,
  release:	hello_close,
  poll:	hello_poll
};

static int __init hello_init(void)
{
  int res;
  struct device* err;

  res = register_chrdev(major_num, DEVICE_NAME, &hello_fops);
  if(res < 0)
  {
    printk(DEVICE_NAME " a major szam: %d nem elerheto\n", major_num);
    return res;
  }
  if(major_num == 0) major_num = res;

  /* Az udev számára jelzés, hogy hozza létre az eszközállományt. */
  hello_class = class_create(THIS_MODULE, CLASS_NAME);
  err = device_create(hello_class, NULL, MKDEV(major_num, 0), NULL, DEVICE_NAME);
  
  printk(DEVICE_NAME " betoltve. Major szam: %d\n", major_num);
  return 0;
}

static void __exit hello_exit(void)
{
  device_destroy(hello_class, MKDEV(major_num, 0));
  class_unregister(hello_class);
  class_destroy(hello_class);
  
  unregister_chrdev(major_num, DEVICE_NAME);
  printk(DEVICE_NAME " felszabaditva.\n");	
}

module_init(hello_init);
module_exit(hello_exit);

MODULE_DESCRIPTION("Hello driver");
MODULE_LICENSE("GPL");
