#include "contactswindow.h"
#include "contactdialog.h"
#include <QtGui>
#include <QStringBuilder>

ContactsWindow::ContactsWindow(QWidget *parent) :
    QMainWindow(parent)
{
    this->setWindowTitle(tr("Cmlista"));

    fileMenu = this->menuBar()->addMenu(tr("&Fjl"));

    newContactAction = new QAction(tr("&j bejegyzs"), this);
    newContactAction->setShortcut(QKeySequence::New);
    newContactAction->setStatusTip(tr("j bejegyzs felvtele"));
    connect(newContactAction, SIGNAL(triggered()), this, SLOT(addNewContact()));
    fileMenu->addAction(newContactAction);

    showSearchAction = new QAction(tr("Keres&s"), this);
    showSearchAction->setShortcut(QKeySequence::Find);
    showSearchAction->setStatusTip(tr("Keress nv alapjn"));
    fileMenu->addAction(showSearchAction);
    connect(showSearchAction, SIGNAL(triggered()), this, SLOT(showSearch()));

    exitAction = new QAction(tr("&Kilp"), this);
    exitAction->setShortcut(tr("Ctrl+Q"));
    exitAction->setStatusTip(tr("Alkalmazs bezrsa"));
    connect(exitAction, SIGNAL(triggered()), this, SLOT(close()));
    fileMenu->addAction(exitAction);

    this->table = new QTableWidget;
    table->setSelectionMode(QAbstractItemView::SingleSelection);
    table->setColumnCount(2);
    this->setCentralWidget(table);
    QStringList headerLabels;
    headerLabels<<"Nv"<<"E-mail";
    table->setHorizontalHeaderLabels(headerLabels);
    connect(table, SIGNAL(itemSelectionChanged()), this, SLOT(updateStatusText()));

    searchDialog = new SearchDialog;
    connect(searchDialog, SIGNAL(searchInvoked(QString)), this, SLOT(search(QString)));
}

void ContactsWindow::showSearch(){
    if (!searchDialog->isVisible())
        this->searchDialog->show();
}

void ContactsWindow::search(QString expression){
    int currentlySelectedRowNbr = -1;
    QList<QTableWidgetItem*> selectedItems = table->selectedItems();
    if (selectedItems.length() > 0)
        currentlySelectedRowNbr = selectedItems.first()->row();
    int i;
    bool found = false;
    for (i = currentlySelectedRowNbr + 1; i < table->rowCount(); i++) {
        if (table->item(i, 0)->text().contains(expression)) {
            table->clearSelection();
            table->item(i, 0)->setSelected(true);
            found = true;
            break;
        }
    }
    if (!found){
        table->clearSelection();
    }
}

void ContactsWindow::closeEvent(QCloseEvent *event)
{
    QMessageBox msg(this);
    msg.setWindowTitle(tr("Kilps"));
    msg.setText(tr("Tnyleg kilp?"));
    msg.setStandardButtons(QMessageBox::Yes | QMessageBox::Cancel);
    if (msg.exec() == QMessageBox::Yes)
        event->accept();
    else
        event->ignore();
}

void ContactsWindow::addNewContact(){
    ContactDialog * newContactDialog = new ContactDialog(this);
    if (newContactDialog->exec() == QDialog::Accepted)
    {
        int currentRowsNbr = table->rowCount();
        table->insertRow(currentRowsNbr);
        QTableWidgetItem * newNameItem = new QTableWidgetItem(newContactDialog->getName());
        table->setItem(currentRowsNbr, 0, newNameItem);
        QTableWidgetItem * newEmailitem = new QTableWidgetItem(newContactDialog->getEmail());
        table->setItem(currentRowsNbr, 1, newEmailitem);
    }
    delete newContactDialog;
}

void ContactsWindow::updateStatusText()
{
    QList<QTableWidgetItem*> selectedItems = table->selectedItems();
    if (selectedItems.length() > 0)
        this->statusBar()->showMessage(
            "Name: " %
            table->item(selectedItems.first()->row(), 0)->text()
                );
}
