#include "datasource.h"
#include "document.h"
#include <QtGui>
#include <QtCore>

Document::Document(QObject *parent) :
    QObject(parent)
{
    state = new QList<double>();
    views = new QList<View*>;
}

void Document::addView(View *view){
    this->views->append(view);
}

void Document::removeView(View * view)
{
    this->views->removeAll(view);
}

QList<double> * Document::getState() {
    return this->state;
}

void Document::appendData(double value){
    this->state->append(value);
    updateAllViews();
}

bool Document::openFile(QString fileName){
    this->filename = fileName;
    QFile file(fileName);
    if (!file.open(QIODevice::ReadOnly | QIODevice::Text))
        return false;
    QTextStream in(&file);
    while (!in.atEnd()) {
        QString valueStr = in.readLine();
        double value = valueStr.toDouble();
        this->state->append(value);
    }

    updateAllViews();
    return true;
}

bool Document::saveFile(){
    QFile file(filename);
    if (!file.open(QIODevice::WriteOnly | QIODevice::Text))
        return false;
    QTextStream out(&file);
    QListIterator<double> iterator(*state);
    while (iterator.hasNext()){
        out << iterator.next() << "\n";
    }
    return true;
}



void Document::updateAllViews() {
    QListIterator<View*> iterator(*views);
    while(iterator.hasNext())
    {
        iterator.next()->updateView();
    }
}



