#include "mdimainwindow.h"
#include "tableview.h"
#include "diagramview.h"
#include "datasource.h"
#include "documentwindow.h"

MDIMainWindow::MDIMainWindow(QWidget *parent) :
    QMainWindow(parent)
{
    mdiArea = new QMdiArea;
    this->setCentralWidget(mdiArea);

    documents = new QList<Document*>;

    fileMenu = this->menuBar()->addMenu(tr("&File"));
    fileToolbar = this->addToolBar(tr("File"));

    openDocumentAction = new QAction(tr("&Open"), this);
    openDocumentAction->setIcon(QIcon::fromTheme("document-open"));
    openDocumentAction->setShortcut(QKeySequence::Open);
    fileMenu->addAction(openDocumentAction);
    fileToolbar->addAction(openDocumentAction);
    connect(openDocumentAction, SIGNAL(triggered()), this, SLOT(open()));

    saveDocumentAction = new QAction(tr("&Save"), this);
    saveDocumentAction->setIcon(QIcon::fromTheme("document-save"));
    saveDocumentAction->setShortcut(QKeySequence::Close);
    fileMenu->addAction(saveDocumentAction);
    fileToolbar->addAction(saveDocumentAction);
    connect(saveDocumentAction, SIGNAL(triggered()), this, SLOT(save()));

    attachDataSourceAction = new QAction(tr("&Attach"), this);
    attachDataSourceAction->setIcon(QIcon::fromTheme("system-run"));
    fileToolbar->addAction(attachDataSourceAction);
    connect(attachDataSourceAction, SIGNAL(triggered()), this, SLOT(attachSource()));

    exitAction = new QAction(tr("E&xit"), this);
    exitAction->setIcon(QIcon::fromTheme("application-exit"));
    exitAction->setShortcut(QKeySequence::Quit);
    fileMenu->addAction(exitAction);
    connect(exitAction, SIGNAL(triggered()), this, SLOT(close()));
}

void MDIMainWindow::open() {
    QString fileName = QFileDialog::getOpenFileName(this,
        tr("Open document"), "~/Documents", tr("Document files (*.data)"));

    if (fileName == "")
        return;

    Document * doc = new Document;
    this->documents->append(doc);
    doc->openFile(fileName);

    DocumentWindow * w = new DocumentWindow;
    w->setObjectName("DocumentWindow");
    QMdiSubWindow * wContainer = this->mdiArea->addSubWindow(w);
    w->setDocument(doc);
    wContainer->setWindowTitle(fileName);

    TableView * table = new TableView;
    table->setDocument(doc);
    w->addView(table);
    doc->addView(table);

    DiagramView * diagram = new DiagramView;
    diagram->setDocument(doc);
    w->addView(diagram);
    doc->addView(diagram);

     w->show();
}

void MDIMainWindow::save(){
    QMdiSubWindow * subWindow = this->mdiArea->activeSubWindow();
    if (subWindow == NULL)
        return;
    DocumentWindow * w = subWindow->findChild<DocumentWindow *>("DocumentWindow");
    if (w == NULL)
        return;
    w->getDocument()->saveFile();
}


void MDIMainWindow::attachSource(){
    QMdiSubWindow * subWindow = this->mdiArea->activeSubWindow();
    if (subWindow == NULL)
        return;
    DocumentWindow * w = subWindow->findChild<DocumentWindow *>("DocumentWindow");
    if (w == NULL)
        return;
    if (w->getDataSource() != NULL)
        return;

    DataSource * source = new DataSource;
    source->setDocument(w->getDocument());
    w->setDataSource(source);
}
